@@ -5,6 +5,7 @@ package Test::Utils;
 use strict;
 use warnings;
 
+use Carp qw(confess);
 use FindBin;
 use File::Path qw(make_path);
 use File::Spec;
@@ -17,22 +18,29 @@ use YAML::Any qw(Dump);
 use Exporter qw(import);
 our @EXPORT_OK = qw(
   Dump
-  init_logger
-  $OFF $FATAL $WARN $ERROR $INFO $DEBUG $TRACE
+  init_logger $OFF $FATAL $WARN $ERROR $INFO $DEBUG $TRACE
+  find_test_caller
   debug_on debug_off
-  is_array is_hash is_valid_n is_valid
+  is_array is_hash is_valid_n is_valid is_not_valid is_deeply_tied
 );
 our %EXPORT_TAGS = (all => [ @EXPORT_OK ]);
 
-# if you want your own logger, specify the logger config file in GOOGLE_RESTAPI_LOGGER env var.
-# else logger will be turned off.
+# if you pass a logger level, it will be used with easy_init and ignore GOOGLE_RESTAPI_LOGGER.
+# if you want your own logger, specify the logger config file in GOOGLE_RESTAPI_LOGGER env var
+# and pass nothing to this routine. if no level and no env var set, do nothing.
+# levels are: $OFF $FATAL $WARN $ERROR $INFO $DEBUG $TRACE. all these log4perl scalars
+# scalars are expored above.
 sub init_logger {
+  my $level = shift || $ENV{GOOGLE_RESTAPI_LOG_LEVEL};
   my $logger_conf = $ENV{GOOGLE_RESTAPI_LOGGER};
+
   if ($logger_conf) {
+    die "'$logger_conf' is not a readable file" unless -f $logger_conf;
     Log::Log4perl->init($logger_conf);
-  } else {
-    Log::Log4perl->easy_init(shift || $OFF);
+  } elsif ($level) {
+    Log::Log4perl->easy_init($level);
   }
+
   return;
 }
 
@@ -47,10 +55,34 @@ sub log_file_name {
   make_path($logdir);
 
   my $logpath = File::Spec->catfile($logdir, $logfile);
-  # warn "File logging will be sent to $logpath\n";
+  warn "File logging will be sent to $logpath\n";
   return $logpath;
 }
 
+# used for building/reading exchanges for unit testing. finds the calling
+# subroutine under the testing framework to associate with the exchange. would
+# normally be just the T::G::RestApi caller, but to support mock spreadsheets,
+# also check Test::Unit::TestBase::startup|shutdown.
+sub find_test_caller {
+  my $test_base_method;
+  my $test_child_method;
+  for (0..30) {
+    my ($package, undef, undef, $subroutine) = caller($_);
+    last unless $package;
+    next unless $subroutine =~ /^(Test::Google::RestApi::|Test::Unit::TestBase::startup|Test::Unit::TestBase::shutdown)/;
+    if ($subroutine =~ /^Test::Unit::TestBase::/) {
+      $test_base_method = $subroutine;
+    } else {
+      $test_child_method = $subroutine;
+      last;
+    }
+  }
+  my $test_method = $test_child_method || $test_base_method;
+  confess "Unable to locate test subroutine to find exchanges" unless $test_method;
+
+  return $test_child_method || $test_base_method;
+}
+
 # call these to temporarily toggle debug-level logger messages around particular tests so you
 # can see internally what's going on within the framework.
 sub debug_on  { Log::Log4perl->get_logger('')->level($DEBUG); }
@@ -77,16 +109,34 @@ sub is_valid {
   $test_name = pop if !ref($_[-1]);
   $test_name .= ' passes validation' if $test_name;
 
-  my ($array, @validation) = @_;
+  my ($values, @validation) = @_;
 
+  my @result;
   try {
-    $array = $array->() if ref($array) eq 'CODE';
-    validate([$array], @validation);
+    $values = $values->() if ref($values) eq 'CODE';
+    @result = validate([$values], @validation);
     pass $test_name;
   } catch {
     my $err = $_;
     fail "$test_name: $err";
   };
+  return @result;
+}
+
+sub is_not_valid {
+  my $test_name = '';
+  $test_name = pop if !ref($_[-1]);
+  $test_name .= ' does not pass validation' if $test_name;
+
+  my ($values, @validation) = @_;
+
+  try {
+    $values = $values->() if ref($values) eq 'CODE';
+    validate([$values], @validation);
+    fail $test_name;
+  } catch {
+    pass $test_name;
+  };
   return;
 }
 
@@ -108,4 +158,11 @@ sub is_valid_n {
   return;
 }
 
+sub is_deeply_tied {
+  my ($got, $expected, $title) = @_;
+  my %got;
+  @got{ keys %$got } = values %$got;
+  return is_deeply \%got, $expected, $title;
+}
+
 1;
